import math
import os
from unified_planning.shortcuts import *
# from planning_tests.commons.problem import TestCaseProblem

FILE_PATH = os.path.dirname(os.path.abspath(__file__))


class TampLogistics01: # (TestCaseProblem):

    def __init__(self, expected_version):
        pass
        # TestCaseProblem.__init__(self, expected_version)


    def get_problem(self):
        t_forklift = MovableType("forklift")
        t_truck = MovableType("truck")

        occ_map = OccupancyMap(os.path.join(FILE_PATH, "maps", "logistics-map-1.yaml"), (0, 0))

        t_robot_config = ConfigurationType("robot_config", occ_map, 3)
        t_container = UserType("container")
        t_slot = UserType("slot")

        forklift_at = Fluent("forklift_at", BoolType(), robot=t_forklift, configuration=t_robot_config)
        truck_at = Fluent("truck_at", BoolType(), robot=t_truck, configuration=t_robot_config)
        container_at = Fluent("container_at", BoolType(), container=t_container, configuration=t_robot_config)
        forklift_carries = Fluent("forklift_cargo", BoolType(), robot=t_forklift, container=t_container)

        truck_carries = Fluent("truck_cargo", BoolType(), robot=t_truck, t_slot=t_slot, container=t_container)

        pick1 = ConfigurationObject("pick-up-1", t_robot_config, (46.0, 26.0, 3*math.pi/2))
        pick2 = ConfigurationObject("pick-up-2", t_robot_config, (40.0, 26.0, 3*math.pi/2))

        loading1 = ConfigurationObject("loading-1", t_robot_config, (4.0, 4.0, 3*math.pi/2))

        delivery1 = ConfigurationObject("office-8", t_robot_config, (32.0, 24.0, math.pi/2))

        fl1 = MovableObject(
            "forklift-1",
            t_forklift,
            footprint=[(-1.0, 0.5), (1.0, 0.5), (1.0, -0.5), (-1.0, -0.5)],
            motion_model=MotionModels.REEDSSHEPP,
            parameters={"turning_radius": 2.0},
        )

        t1 = MovableObject(
            "delivery-truck-1",
            t_truck,
            footprint=[(-1.0, 0.5), (2.0, 0.5), (2.0, -0.5), (-1.0, -0.5)],
            motion_model=MotionModels.REEDSSHEPP,
            parameters={"turning_radius": 4.0},
        )

        nothing = Object("nothing", t_container)
        c1 = Object("container-1", t_container)
        c2 = Object("container-2", t_container)

        slot1 = Object("slot-1", t_slot)
        slot2 = Object("slot-2", t_slot)
        slot3 = Object("slot-3", t_slot)

        def move_forklift():
            action = InstantaneousMotionAction(
                "move-forklift", forklift=t_forklift, c_from=t_robot_config, c_to=t_robot_config
            )
            r = action.parameter("forklift")
            a = action.parameter("c_from")
            b = action.parameter("c_to")
            action.add_precondition(forklift_at(r, a))
            action.add_effect(forklift_at(r, a), False)
            action.add_effect(forklift_at(r, b), True)
            #action.add_motion_constraint(Waypoints(r, a, [b]))
            return action

        def move_truck():
            action = InstantaneousMotionAction(
                "move-truck", truck=t_truck, c_from=t_robot_config, c_to=t_robot_config
            )
            r = action.parameter("truck")
            a = action.parameter("c_from")
            b = action.parameter("c_to")
            action.add_precondition(truck_at(r, a))
            action.add_effect(truck_at(r, a), False)
            action.add_effect(truck_at(r, b), True)
            #action.add_motion_constraint(Waypoints(r, a, [b]))
            return action

        def pick_up():
            action = InstantaneousMotionAction(
                "pick", robot=t_forklift, loc=t_robot_config, container=t_container
            )
            r = action.parameter("robot")
            loc = action.parameter("loc")
            c = action.parameter("container")
            action.add_precondition(forklift_at(r, loc))
            action.add_precondition(container_at(c, loc))
            action.add_precondition(forklift_carries(r, nothing))
            action.add_precondition(Not(forklift_carries(r, c)))
            action.add_effect(forklift_carries(r, c), True)
            action.add_effect(container_at(c, loc), False)
            action.add_effect(forklift_carries(r, nothing), False)
            return action

        def load_on_truck():
            action = InstantaneousMotionAction(
                "load-truck", forklift=t_forklift, truck=t_truck, slot=t_slot, loc=t_robot_config, container=t_container
            )
            forklift = action.parameter("forklift")
            truck = action.parameter("truck")
            slot = action.parameter("slot")
            loc = action.parameter("loc")
            container = action.parameter("container")

            action.add_precondition(forklift_at(forklift, loc))
            action.add_precondition(truck_at(truck, loc))
            action.add_precondition(forklift_carries(forklift, container))
            action.add_precondition(truck_carries(truck, slot, nothing))

            action.add_effect(forklift_carries(forklift, container), False)
            action.add_effect(truck_carries(truck, slot, container), True)
            return action

        def dump_from_truck():
            action = InstantaneousMotionAction(
                "unload-truck", truck=t_truck, slot=t_slot, loc=t_robot_config, container=t_container
            )
            truck = action.parameter("truck")
            slot = action.parameter("slot")
            loc = action.parameter("loc")
            container = action.parameter("container")

            action.add_precondition(truck_at(truck, loc))
            action.add_precondition(truck_carries(truck, slot, container))
            action.add_effect(truck_carries(truck, slot, container), False)
            action.add_effect(container_at(container, loc), True)
            return action


        problem = Problem("robot")
        problem.add_fluent(forklift_at, default_initial_value=False)
        problem.add_fluent(truck_at, default_initial_value=False)
        problem.add_fluent(container_at, default_initial_value=False)
        problem.add_fluent(forklift_carries, default_initial_value=False)
        problem.add_fluent(truck_carries, default_initial_value=False)

        problem.add_action(move_forklift())
        problem.add_action(move_truck())
        problem.add_action(pick_up())
        problem.add_action(load_on_truck())
        problem.add_action(dump_from_truck())

        problem.add_object(pick1)
        problem.add_object(pick2)
        problem.add_object(loading1)
        problem.add_object(delivery1)

        problem.add_object(fl1)
        problem.add_object(t1)

        problem.add_object(nothing)
        problem.add_object(c1)
        problem.add_object(c2)

        problem.add_object(slot1)
        problem.add_object(slot2)
        problem.add_object(slot3)

        problem.set_initial_value(forklift_carries(fl1, nothing), True)
        problem.set_initial_value(truck_carries(t1, slot1, nothing), True)
        problem.set_initial_value(truck_carries(t1, slot2, nothing), True)
        problem.set_initial_value(truck_carries(t1, slot3, nothing), True)

        problem.set_initial_value(container_at(c1, pick1), True)
        problem.set_initial_value(container_at(c2, pick2), True)

        problem.set_initial_value(forklift_at(fl1, pick1), True)
        problem.set_initial_value(truck_at(t1, loading1), True)

        problem.add_goal(container_at(c1, delivery1))
        problem.add_goal(container_at(c2, delivery1))

        return problem

    def get_description(self):
        return 'Moving a robot in a simple office map'

    def version(self):
        return 1



import math
import os
from unified_planning.shortcuts import *
# from planning_tests.commons.problem import TestCaseProblem

FILE_PATH = os.path.dirname(os.path.abspath(__file__))


class TampOffice03: # (TestCaseProblem):

    def __init__(self, expected_version):
        pass
        # TestCaseProblem.__init__(self, expected_version)


    def get_problem(self):
        t_robot = MovableType("robot")

        occ_map = OccupancyMap(os.path.join(FILE_PATH, "maps", "office-map-1.yaml"), (0, 0))

        t_robot_config = ConfigurationType("robot_config", occ_map, 3)
        t_parcel = UserType("parcel")

        robot_at = Fluent("robot_at", BoolType(), robot=t_robot, configuration=t_robot_config)
        parcel_at = Fluent("parcel_at", BoolType(), parcel=t_parcel, configuration=t_robot_config)
        carries = Fluent("carries", BoolType(), robot=t_robot, parcel=t_parcel)

        park1 = ConfigurationObject("parking-1", t_robot_config, (46.0, 26.0, 3*math.pi/2))
        park2 = ConfigurationObject("parking-2", t_robot_config, (40.0, 26.0, 3*math.pi/2))

        office1 = ConfigurationObject("office-1", t_robot_config, (4.0, 4.0, 3*math.pi/2))
        office2 = ConfigurationObject("office-2", t_robot_config, (14.0, 4.0, math.pi/2))
        office3 = ConfigurationObject("office-3", t_robot_config, (24.0, 4.0, 3*math.pi/2))
        office4 = ConfigurationObject("office-4", t_robot_config, (32.0, 4.0, 3*math.pi/2))
        office5 = ConfigurationObject("office-5", t_robot_config, (4.0, 24.0, 3*math.pi/2))
        office6 = ConfigurationObject("office-6", t_robot_config, (14.0, 24.0, math.pi/2))
        office7 = ConfigurationObject("office-7", t_robot_config, (24.0, 24.0, math.pi/2))
        office8 = ConfigurationObject("office-8", t_robot_config, (32.0, 24.0, math.pi/2))

        r1 = MovableObject(
            "r1",
            t_robot,
            footprint=[(-1.0, 0.5), (1.0, 0.5), (1.0, -0.5), (-1.0, -0.5)],
            motion_model=MotionModels.REEDSSHEPP,
            parameters={"turning_radius": 2.0},
        )

        r2 = MovableObject(
            "r2",
            t_robot,
            footprint=[(-1.0, 0.5), (1.0, 0.5), (1.0, -0.5), (-1.0, -0.5)],
            motion_model=MotionModels.REEDSSHEPP,
            parameters={"turning_radius": 2.0},
        )

        nothing = Object("nothing", t_parcel)
        p1 = Object("parcel-1", t_parcel)
        p2 = Object("parcel-2", t_parcel)

        move = InstantaneousMotionAction(
            "move", robot=t_robot, c_from=t_robot_config, c_to=t_robot_config
        )
        robot = move.parameter("robot")
        c_from = move.parameter("c_from")
        c_to = move.parameter("c_to")
        move.add_precondition(robot_at(robot, c_from))
        move.add_effect(robot_at(robot, c_from), False)
        move.add_effect(robot_at(robot, c_to), True)
        move.add_motion_constraint(Waypoints(robot, c_from, [c_to]))

        pick = InstantaneousMotionAction(
            "pick", robot=t_robot, loc=t_robot_config, parcel=t_parcel
        )
        pick_robot = pick.parameter("robot")
        pick_loc = pick.parameter("loc")
        pick_parcel = pick.parameter("parcel")
        pick.add_precondition(robot_at(pick_robot, pick_loc))
        pick.add_precondition(parcel_at(pick_parcel, pick_loc))
        pick.add_precondition(carries(pick_robot, nothing))
        pick.add_precondition(Not(carries(pick_robot, pick_parcel)))
        pick.add_effect(carries(pick_robot, pick_parcel), True)
        pick.add_effect(parcel_at(pick_parcel, pick_loc), False)
        pick.add_effect(carries(pick_robot, nothing), False)


        place = InstantaneousMotionAction(
            "place", robot=t_robot, loc=t_robot_config, parcel=t_parcel
        )
        place_robot = place.parameter("robot")
        place_loc = place.parameter("loc")
        place_parcel = place.parameter("parcel")
        place.add_precondition(robot_at(place_robot, place_loc))
        place.add_precondition(carries(place_robot, place_parcel))
        place.add_precondition(Not(parcel_at(place_parcel, place_loc)))
        place.add_precondition(Not(carries(place_robot, nothing)))
        place.add_effect(carries(place_robot, place_parcel), False)
        place.add_effect(carries(place_robot, nothing), True)
        place.add_effect(parcel_at(place_parcel, place_loc), True)



        problem = Problem("robot")
        problem.add_fluent(robot_at, default_initial_value=False)
        problem.add_fluent(parcel_at, default_initial_value=False)
        problem.add_fluent(carries, default_initial_value=False)
        problem.add_action(move)
        problem.add_action(pick)
        problem.add_action(place)

        problem.add_object(park1)
        problem.add_object(park2)
        problem.add_object(office1)
        problem.add_object(office2)
        problem.add_object(office3)
        problem.add_object(office4)
        problem.add_object(office5)
        problem.add_object(office6)
        problem.add_object(office7)
        problem.add_object(office8)

        problem.add_object(r1)
        problem.add_object(r2)

        problem.add_object(nothing)
        problem.add_object(p1)
        problem.add_object(p2)

        problem.set_initial_value(carries(r1, nothing), True)
        problem.set_initial_value(carries(r2, nothing), True)

        problem.set_initial_value(parcel_at(p1, office1), True)
        problem.set_initial_value(parcel_at(p2, office6), True)

        problem.set_initial_value(robot_at(r1, park1), True)
        problem.set_initial_value(robot_at(r2, park2), True)

        problem.add_goal(robot_at(r1, park1))
        problem.add_goal(robot_at(r2, park2))
        problem.add_goal(parcel_at(p1, office2))
        problem.add_goal(parcel_at(p2, office3))

        return problem

    def get_description(self):
        return 'Moving a robot in a simple office map'

    def version(self):
        return 1



import math
import os
from unified_planning.shortcuts import *
# from planning_tests.commons.problem import TestCaseProblem

FILE_PATH = os.path.dirname(os.path.abspath(__file__))


class TampConstruction01: # (TestCaseProblem):

    def __init__(self, expected_version):
        pass
        # TestCaseProblem.__init__(self, expected_version)


    def get_problem(self):
        t_robot = MovableType("robot")

        occ_map = OccupancyMap(os.path.join(FILE_PATH, "maps", "simple-construction-map.yaml"), (0, 0))

        t_robot_config = ConfigurationType("robot_config", occ_map, 3)
        t_material = UserType("material")

        robot_at = Fluent("robot_at", BoolType(), robot=t_robot, configuration=t_robot_config)
        material_at = Fluent("material_at", BoolType(), material=t_material, configuration=t_robot_config)
        provider = Fluent("provider", BoolType(), locaction=t_robot_config, material=t_material)
        consumer = Fluent("consumer", BoolType(), locaction=t_robot_config, material=t_material)
        loaded = Fluent("loaded", BoolType(), robot=t_robot, material=t_material)
        connected = Fluent("connected", BoolType(), loc_a=t_robot_config, loc_b=t_robot_config)
        occupied = Fluent("occupied", BoolType(), loc=t_robot_config)

        queue1 = ConfigurationObject("queue-1", t_robot_config, (10.306, 46.206, -3.111))
        queue2 = ConfigurationObject("queue-2", t_robot_config, (14.120, 46.213, -3.136))
        queue3 = ConfigurationObject("queue-3", t_robot_config, (17.606, 42.904, 2.245))

        dump1 = ConfigurationObject("dump-1", t_robot_config, (39.724, 37.334, 2.344))

        park1 = ConfigurationObject("parking-1", t_robot_config, (34.935, 4.191, 1.542))
        park2 = ConfigurationObject("parking-2", t_robot_config, (40.102, 4.374, 1.539))
        park3 = ConfigurationObject("parking-3", t_robot_config, (44.537, 4.068, 1.540))

        truck1 = MovableObject(
            "truck1",
            t_robot,
            footprint=[(-1.0, 0.5), (1.0, 0.5), (1.0, -0.5), (-1.0, -0.5)],
            motion_model=MotionModels.REEDSSHEPP,
            parameters={"turning_radius": 2.0},
        )

        truck2 = MovableObject(
            "truck2",
            t_robot,
            footprint=[(-1.0, 0.5), (1.0, 0.5), (1.0, -0.5), (-1.0, -0.5)],
            motion_model=MotionModels.REEDSSHEPP,
            parameters={"turning_radius": 2.0},
        )

        truck3 = MovableObject(
            "truck3",
            t_robot,
            footprint=[(-1.0, 0.5), (1.0, 0.5), (1.0, -0.5), (-1.0, -0.5)],
            motion_model=MotionModels.REEDSSHEPP,
            parameters={"turning_radius": 2.0},
        )

        nothing = Object("nothing", t_material)
        m1 = Object("sand", t_material)
        m2 = Object("cement", t_material)
        m3 = Object("planks", t_material)

        move = InstantaneousMotionAction("move", robot=t_robot, c_from=t_robot_config, c_to=t_robot_config)
        robot = move.parameter("robot")
        c_from = move.parameter("c_from")
        c_to = move.parameter("c_to")
        move.add_precondition(robot_at(robot, c_from))
        move.add_precondition(connected(c_from, c_to))
        move.add_precondition(occupied(c_from))
        move.add_precondition(Not(occupied(c_to)))
        move.add_effect(robot_at(robot, c_from), False)
        move.add_effect(robot_at(robot, c_to), True)
        move.add_effect(occupied(c_from), False)
        move.add_effect(occupied(c_to), True)

        move.add_motion_constraint(Waypoints(robot, c_from, [c_to]))

        load = InstantaneousMotionAction("load", robot=t_robot, loc=t_robot_config, material=t_material)
        load_robot = load.parameter("robot")
        load_loc = load.parameter("loc")
        load_material = load.parameter("material")
        load.add_precondition(robot_at(load_robot, load_loc))
        load.add_precondition(provider(load_loc, load_material))
        load.add_precondition(loaded(load_robot, nothing))
        load.add_precondition(Not(loaded(load_robot, load_material)))
        load.add_effect(loaded(load_robot, load_material), True)
        load.add_effect(loaded(load_robot, nothing), False)


        dump = InstantaneousMotionAction("dump", robot=t_robot, loc=t_robot_config, material=t_material)
        dump_robot = dump.parameter("robot")
        dump_loc = dump.parameter("loc")
        dump_material = dump.parameter("material")
        dump.add_precondition(robot_at(dump_robot, dump_loc))
        dump.add_precondition(consumer(dump_loc, dump_material))
        dump.add_precondition(loaded(dump_robot, dump_material))
        dump.add_precondition(Not(loaded(dump_robot, nothing)))
        dump.add_precondition(Not(material_at(dump_material, dump_loc)))
        dump.add_effect(loaded(dump_robot, dump_material), False)
        dump.add_effect(loaded(dump_robot, nothing), True)
        dump.add_effect(material_at(dump_material, dump_loc), True)

        problem = Problem("construction-test-1")
        problem.add_fluent(robot_at, default_initial_value=False)
        problem.add_fluent(material_at, default_initial_value=False)
        problem.add_fluent(loaded, default_initial_value=False)
        problem.add_fluent(connected, default_initial_value=False)
        problem.add_fluent(provider, default_initial_value=False)
        problem.add_fluent(occupied, default_initial_value=False)
        problem.add_fluent(consumer, default_initial_value=False)

        problem.add_action(move)
        problem.add_action(load)
        problem.add_action(dump)

        problem.add_object(park1)
        problem.add_object(park2)
        problem.add_object(park3)

        problem.add_object(dump1)

        problem.add_object(queue1)
        problem.add_object(queue2)
        problem.add_object(queue3)

        problem.add_object(truck1)
        problem.add_object(truck2)
        problem.add_object(truck3)

        problem.add_object(nothing)
        problem.add_object(m1)
        problem.add_object(m2)
        problem.add_object(m3)

        problem.set_initial_value(connected(queue1, park1), True)
        problem.set_initial_value(connected(queue1, park2), True)
        problem.set_initial_value(connected(queue1, park3), True)
        problem.set_initial_value(connected(queue1, queue3), True)
        problem.set_initial_value(connected(queue1, dump1), True)

        problem.set_initial_value(connected(dump1, park1), True)
        problem.set_initial_value(connected(dump1, park2), True)
        problem.set_initial_value(connected(dump1, park3), True)
        problem.set_initial_value(connected(dump1, queue3), True)

        problem.set_initial_value(connected(park1, dump1), True)
        problem.set_initial_value(connected(park1, queue3), True)
        problem.set_initial_value(connected(park2, dump1), True)
        problem.set_initial_value(connected(park2, queue3), True)
        problem.set_initial_value(connected(park3, dump1), True)
        problem.set_initial_value(connected(park3, queue3), True)

        problem.set_initial_value(connected(queue2, queue1), True)
        problem.set_initial_value(connected(queue3, queue2), True)

        problem.set_initial_value(loaded(truck1, nothing), True)
        problem.set_initial_value(loaded(truck2, nothing), True)
        problem.set_initial_value(loaded(truck3, nothing), True)

        problem.set_initial_value(provider(queue1, m1), True)
        problem.set_initial_value(provider(queue1, m2), True)
        problem.set_initial_value(provider(queue1, m3), True)

        problem.set_initial_value(consumer(dump1, m1), True)
        problem.set_initial_value(consumer(dump1, m2), True)
        problem.set_initial_value(consumer(dump1, m3), True)

        problem.set_initial_value(robot_at(truck1, park1), True)
        #problem.set_initial_value(robot_at(truck2, park2), True)
        #problem.set_initial_value(robot_at(truck3, park3), True)

        problem.set_initial_value(occupied(park1), True)
        problem.set_initial_value(occupied(park2), True)
        problem.set_initial_value(occupied(park3), True)

        problem.add_goal(robot_at(truck1, park1))
        #problem.add_goal(robot_at(truck2, park2))
        #problem.add_goal(robot_at(truck3, park3))

        problem.add_goal(material_at(m1, dump1))
        #problem.add_goal(material_at(m2, dump1))
        #problem.add_goal(material_at(m3, dump1))

        return problem

    def get_description(self):
        return 'Moving a robot in a simple office map'

    def version(self):
        return 1
